function [Collision Distance Location] = BresenhamLine(x0in, x1in, y0in, y1in,Map)
    %  This is a crude, non-optimized function to check for collision
    %  between a laser ray and a bitmap of the world.
    %  function [Collision Distance Location] = BresenhamLine(x0in, x1in, y0in, y1in,Map)
    %  Input:
    %     x0in, y0in : location of the start of the ray (i.e. robot location)
    %     x1in, y1in : location of the end of the ray 
    %  Output:
    %     Collision : true/false if collision between ray and world
    %     Distance  : distance to collision
    %     Location  : world coordinate of the collision, for debug purposes.
    %
    % (c) Philippe Giguere 2010
    %     philippe.giguere@ift.ulaval.ca
     xOrig = x0in;
     yOrig = y0in;
     [sx sy] = size(Map);
     steep = (abs(y1in - y0in) > abs(x1in - x0in));
     Collision = false;
     Distance = 1e10;
     Location = [-1 -1];
     xstep = 1;
     toto = false;
     
     if steep
         x0 = y0in;
         y0 = x0in;
         x1 = y1in;
         y1 = x1in;
     else
         x0 = x0in;
         y0 = y0in;
         x1 = x1in;
         y1 = y1in;
     end
     
     if (x0 > x1)
         tmp = x1;
         x1 = x0;
         x0 = tmp;
         tmp = y1;
         y1 = y0;
         y0 = tmp;
     end
     deltax = x1 - x0;
     deltay = abs(y1 - y0);
     errorLine = deltax / 2;

     y = y0;
     if (y0 < y1)
         ystep = 1;
     else
         ystep = -1;
     end
     
     for x=x0:x1
         if steep
             xCheck = y;
             yCheck = x;
         else
             xCheck = x;
             yCheck = y;
         end
         if ((xCheck >= 1) && (round(xCheck) <= sy) && (yCheck >= 1) && (round(yCheck) < sx))
             if (toto)
                 hold on; plot(xCheck,yCheck,'s');
             end
             if (Map(round(yCheck),round(xCheck)) <128) % Image are stored fliped diagonally
                 Collision = true;
                 TmpDistance = sqrt((xOrig-xCheck)^2+(yOrig-yCheck)^2);
                 if (TmpDistance < Distance)
                     Distance = TmpDistance;
                     Location = [xCheck yCheck];
                 end
             end
         end
         errorLine = errorLine - deltay;
         if errorLine < 0
             y = y + ystep;
             errorLine = errorLine + deltax;
         end
     end
 end
         
