function [xArray yArray] = BresenhamLineArray(x0in, x1in, y0in, y1in)
    %  This is a crude, non-optimized function to return all the grid
    %  coordinates traversed by a laser beam.
    %  function [xArray yArray] = BresenhamLineArray(x0in, x1in, y0in, y1in)
    %  Input:
    %     x0in, y0in : location of the start of the ray (i.e. robot location)
    %     x1in, y1in : location of the end of the ray 
    %  Output:
    %     xArray : grid location, X coordinates
    %     yArray : grid location, X coordinates
    %
    % (c) Philippe Giguere 2013
    %     philippe.giguere@ift.ulaval.ca
     xOrig = x0in;
     yOrig = y0in;
     xArray = [];
     yArray = [];
     steep = (abs(y1in - y0in) > abs(x1in - x0in));
     Distance = 1e10;
     Location = [-1 -1];
     xstep = 1;
     toto = false;
     
     if steep
         x0 = y0in;
         y0 = x0in;
         x1 = y1in;
         y1 = x1in;
     else
         x0 = x0in;
         y0 = y0in;
         x1 = x1in;
         y1 = y1in;
     end
     
     if (x0 > x1)
         tmp = x1;
         x1 = x0;
         x0 = tmp;
         tmp = y1;
         y1 = y0;
         y0 = tmp;
     end
     deltax = x1 - x0;
     deltay = abs(y1 - y0);
     errorLine = deltax / 2;

     y = y0;
     if (y0 < y1)
         ystep = 1;
     else
         ystep = -1;
     end
     
     for x=x0:x1
         if steep
             xCheck = y;
             yCheck = x;
         else
             xCheck = x;
             yCheck = y;
         end
         xArray = [round(xCheck) xArray];
         yArray = [round(yCheck) yArray];
         errorLine = errorLine - deltay;
         if errorLine < 0
             y = y + ystep;
             errorLine = errorLine + deltax;
         end
     end
 end
         
