function [ScanLine] = GetLaserScan(xRobot,yRobot,Heading,WorldMap,LRF)
    % function [ScanLine] = GetLaserScan(xRobot,yRobot,Heading,WorldMap,LRF)
    % where xRobot: x position of the robot
    %       yRobot: y position of the robot
    %       Heading: heading of the robot, in rad. 0 rad is x-axis
    %       WorldMap : bitmap of the world
    %       LRF : structure containing LRF parameters. 
    % returns:
    %       ScanLine : a matrix containing the angles (rad) and distance.
    %                  if distance < 0, then there was no object detected.
    %
    % (c) Philippe Giguere 2013
    %     philippe.giguere@ift.ulaval.ca
    %
    Draw = false;
    DataIndex = 1;
    for ScanAngle = LRF.StartAngle:LRF.dAngle:LRF.EndAngle
        UsedScanAngle = ScanAngle + Heading;
        xTip = xRobot+LRF.LaserRange*cos(UsedScanAngle);
        yTip = yRobot+LRF.LaserRange*sin(UsedScanAngle);
        if (LRF.Draw)
            hold on;
            h = line([xRobot xTip],[yRobot yTip]);
            set(h,'Color','r');
        end
        [Hit Distance HitLocation] = BresenhamLine(xRobot,xTip,yRobot,yTip,WorldMap);
        if (LRF.Draw)
            plot(HitLocation(1),HitLocation(2),'*');
            hold off;
        end
        ScanLine(1,DataIndex) = ScanAngle;
        if (Hit)
            ScanLine(2,DataIndex) = max(Distance  + randn*LRF.noise,0);
        else
            ScanLine(2,DataIndex) = NaN; %LRF.LaserRange;
        end
        DataIndex = DataIndex + 1;
    end
end
