function map = GridMapping2(pose, scan, LRF)
    global MyGridMap
    
    MaxLaserRange = LRF.LaserRange;
    LaserNoise = LRF.noise;
    PRand = 0.01;
    PZ_NotCX = PRand;
    
    % Update de la carte pour chaque rayon
    for index = 1:size(scan,2)
        if ~isnan(scan(2,index))
            % Scan valide
            xEnd = pose(1) + cos(scan(1,index)+pose(3))*(scan(2,index)+2*LaserNoise);  % pour tenir compte incertitude
            yEnd = pose(2) + sin(scan(1,index)+pose(3))*(scan(2,index)+2*LaserNoise);  % pour tenir compte incertitude
            xUpdate = round(xEnd); yUpdate = round(yEnd);
        else
            % we will have the maximum range as the endpoint, but no
            % obstacles.
            xEnd = pose(1) + cos(scan(1,index)+pose(3))*MaxLaserRange;
            yEnd = pose(2) + sin(scan(1,index)+pose(3))*MaxLaserRange;
            xUpdate = round(xEnd); yUpdate = round(yEnd);
        end
        
        % Get the empty space now
        [xFree yFree] = BresenhamLineArray(pose(1), xUpdate, pose(2),yUpdate);

        %update the grid locations
        Z   = scan(2,index);
        KRand = (1-PRand/Z);
        for indexFree = 1:size(xFree,2)
            xUp = xFree(indexFree);
            yUp = yFree(indexFree);
            r   = sqrt((pose(1)-xUp)^2 + (pose(2)-yUp)^2);
            if (~isnan(Z))
                PZ_CX = KRand*normpdf(r,Z,LaserNoise) + PRand/Z;
                OddX = PZ_CX./PZ_NotCX;
                if (r > Z)
                    if (OddX < 1)
                        OddX = 1; % Behind measurement, Odd should never be less than 1
                    end
                else
                    PZ_CX = PZ_CX ;
                end
            else
                % Cas ou le laser indique out-of-range
                OddX = 1/MaxLaserRange;  % approximatif, a verifier!
            end
            if (~isnan(log(OddX)))
                if ( (xUp <= size(MyGridMap,2)) && (yUp <= size(MyGridMap,1)) && (xUp > 0) && (yUp > 0))
                     MyGridMap(yUp,xUp) = MyGridMap(yUp,xUp) - log(OddX);  % minus sign, because of my convention.
                end
            end
        end
    end
    
    % hCurrentFigure = gcf; % On sauvegarde le handle de la figure en court
    %figure(2);
    subplot(2,2,4);
    imagesc((tanh(MyGridMap.*0.1)+1).*0.5); % To have a better visualization
    set(gca,'YDir','normal'); % just so coordinate won't be upside down.
    %axis equal
    colormap('gray'); caxis([0 1]);
    %drawnow()
    %figure(hCurrentFigure); % On restaure la figure en cours
end